
from typing import List, Optional

from sqlalchemy import (ARRAY, TIMESTAMP, Column, Integer, MetaData, String,
                        Table, create_engine)
from sqlalchemy.orm import registry, sessionmaker

from findontime.records import InsafluFile


class InsafluTables:

    def __init__(self, db_path: str) -> None:
        """
        Setup the database
        """
        self.db_path = db_path

        self.engine = None

    @property
    def is_connected(self) -> bool:
        """
        Check if the database is connected
        """
        return self.engine is not None

    def setup(self):
        self._setup_engine(self.db_path)

        self.insaflu_files = InsafluFilesTable(self.engine)

    def _setup_engine(self, db_path: str):
        """
        Setup the engine for the database"""
        self.engine = create_engine(f"sqlite:///{db_path}")

    def drop_database(self):
        """
        Drop the database
        """

        self.engine.dispose()


class InsafluFilesTable:

    def __init__(self, engine) -> None:
        """
        Setup the database
        """
        self.engine = engine

        self._create_table()

    def _create_table(self):
        """
        Create the table if it doesn't exist"""
        metadata = MetaData()
        mapper_registry = registry(metadata=metadata)
        insaflu_table = Table(
            "insaflu_files",
            mapper_registry.metadata,
            Column("id", Integer, primary_key=True, autoincrement=True),
            Column("sample_id", String),
            Column("barcode", String),
            Column("file_path", String),
            Column("remote_path", String),
            Column("status", Integer),
        )

        mapper_registry.map_imperatively(InsafluFile, insaflu_table)

        metadata.create_all(self.engine)

    def _get_session(self):
        """
        Get a session for the database"""
        Session = sessionmaker(bind=self.engine)
        return Session()

    def _get_sample(self, sample_id: str, barcode: str, file_path: str, remote_path: str) -> Optional[InsafluFile]:
        """
        Get a sample from the database"""

        session = self._get_session()
        sample = session.query(InsafluFile).filter_by(
            sample_id=sample_id, barcode=barcode, file_path=file_path, remote_path=remote_path).first()
        session.close()
        return sample

    def get_sample(self, sample_id: str, barcode: str, file_path: str, remote_path: str) -> Optional[InsafluFile]:
        """
        Get a sample from the database"""

        return self._get_sample(sample_id, barcode, file_path, remote_path)

    def get_sample_by_id(self, sample_id: str) -> Optional[InsafluFile]:
        """
        Get a sample from the database"""

        return self._get_sample(sample_id, "", "", "")

    def add_sample(self, sample: InsafluFile):
        """
        Add a sample to the database"""

        if self.check_sample_in_db(sample):
            return

        session = self._get_session()
        session.add(sample)
        session.commit()
        session.close()

    def check_sample_in_db(self, insaflu_file: InsafluFile) -> bool:
        """
        Check if a sample is in the database"""

        sample = self.get_sample(insaflu_file.sample_id, insaflu_file.barcode,
                                 insaflu_file.file_path, insaflu_file.remote_path)
        return sample is not None



# -*- coding: utf-8 -*-
"""
Created on Fri Feb 17 11:03:42 2023

@author: andre
"""

import os

import pandas as pd
import seaborn as sns

# import matplotlib.pyplot as plt

# CONSTANTS
PLOTTING_OPTIONS = ["jpeg", "png", "html"]


def create_html(out_dir):

    plots = search_folder(out_dir)

    new_html = str("output_plots.html")

    os.chdir(out_dir)

    fh = open(new_html, "w")
    fh.write("<!DOCTYPE html>" + "\n")
    fh.write("<html>" + "\n")
    fh.write("<body>" + "\n")
    fh.write("<h1>Plots testing</h1>" + "\n")
    fh.write("<div class=\"row\">" + "\n")
    fh.write("\t<div class=\"column\">" + "\n")

    for plot in plots:
        image_location = os.path.join(out_dir, plot)

        il1 = f"\t\t<img src=\"{image_location}\""
        il2 = f" alt=\"{plot[:-5]}\" width=\"500\" height=\"400\">"

        image_line = il1+il2

        fh.write(image_line + "\n")
        fh.write("\n")

    fh.write("</body>" + "\n")
    fh.write("</html>" + "\n")
    fh.close()


def search_folder(cwd):
    """
    Lists all the files in the current work directory (cwd).

    Takes:                  Returns:
        str                     list
    """
    cur_files = os.listdir(cwd)
    return cur_files


def get_workflow_num(run_name):
    """
    Gets the workflow code from the run_name.

    Takes:              Returns:
        str*str             str*str
    """
    max_un_index = 0
    for i in range(len(run_name)):
        if run_name[i] == "_":
            max_un_index = i

    max_un_index += 1
    run_num = run_name[max_un_index:]
    return run_num


def add_workflow(df):
    """
    Adds the workflow column to the pandas dataframe and fills it with the
    workflow codes from each run.

    Takes:                          Returns:
        obj: pandas dataframe           obj: pandas dataframe
    """
    df['workflow'] = ''

    for index, row in df.iterrows():
        work_f = row['Run']
        work_f_num = get_workflow_num(work_f)
        df.at[index, 'workflow'] = work_f_num

    return df


def merge_all_reports_and_times(df, metadata_dir):
    """
    Takes the all_reports_file, reads it as a pandas dataframe, 
    gets all the time_elapsed values of each sample and adds it 
    to the pandas dataframe.
    The final dataframe has all the info + time_elapsed.
    """
    # Go to the directory of the metadata files
    os.chdir(metadata_dir)

    # For each metafile, it reads
    meta_files = search_folder(metadata_dir)
    for met_file in meta_files:
        df1 = pd.read_csv(met_file, sep='\t')
        df1 = df1.loc[:, ["sample name", "time elapsed"]]
        # os.chdir("C:\\Users\\andre\\OneDrive - FCT NOVA\André\\Mestrado - Bioinfo\\2º Ano\Projeto em Multi-Ómicas - INSA\\teste_1\\testing_formating_metadata\\new_meta")
        for index, row in df.iterrows():
            if row["Sample"] == df1["sample name"][0]:
                df.at[index, "time elapsed"] = df1.at[0, "time elapsed"]

    # #Add the workflows
    df = add_workflow(df)

    # #Directory of the new all_report file with all the info + time_elapsed
    # cwd="C:\\Users\\andre\\OneDrive - FCT NOVA\André\\Mestrado - Bioinfo\\2º Ano\Projeto em Multi-Ómicas - INSA\\teste_1\\testing_formating_metadata\\new_all_reports"
    # os.chdir(cwd)
    # df.to_csv("new_export_all_reports_13_02_2023.tsv", sep="\t", index=False)

    return df


def plot_s_graphics(pd_df, param, x_axis, y_axis, plot_type, separator, save_or, save_dir, dpi_qual, plot_save_format):

    ans = (True, separator)
    if separator not in pd_df.columns:
        ans = (False, separator)
        separator = None

    # Picking the palette of colors for the plots.
    # pal=sns.color_palette("tab10")

    pd_df = pd_df.sort_values(by=[param, y_axis], ascending=False)

    for par in pd_df[param].unique():
        sub_df = pd_df[pd_df[param] == par]
        sub_df = sub_df.sort_values(by=[param, y_axis], ascending=True)

        # plot_export_name=sub_df

        plot_ = sns.relplot(
            data=sub_df, kind=plot_type,
            x=x_axis, y=y_axis,
            facet_kws=dict(sharex=False), col=param, hue=separator, style=separator,
            markers=True, dashes=True, size=separator)  # ,palette=(pal)

        sns.set(style='whitegrid', rc={'legend.labelspacing': 1.2})

        if save_or == "y":
            plots_dir = os.path.join(save_dir, "output_plots")
            if not os.path.exists(plots_dir):
                os.makedirs(plots_dir)
            os.chdir(plots_dir)

            if plot_save_format == "html":
                plot_file_name = str(param)+"-"+str(par)
                plot_.savefig(f"{param}-{par}.jpeg", dpi=dpi_qual)
                create_html(plots_dir)
            else:
                plot_.savefig(
                    f"{param}-{par}.{plot_save_format}", dpi=dpi_qual)

    if ans[0] == False:
        print(f"{ans[1]} doesnt exist.")

# MAIN FOR COMMAND LINE


def main():
    a = input("> ")
    """
    INSTRUCTIONS:
    
    all_reports file_dir - directory of the all_reports file
        
    metadata_dir - directory of the metadata files
    
    param - plotting for each 'param'
    
    x_axis - column name to assign to x_axis
    
    y_axis - column name to assign to y_axis
    
    plot_type - can be 'scatter' or 'line'
    
    separator - represents the various samples of the same 'param' which have different 'separator'.
    Example: separator = 'workflow' will show the different workflows used for the same same.
    """
    # all_rep_dir=str(input("Input the all_reports.tsv file directory:\n > "))
    # while os.path.exists(all_rep_dir)!=True:
    #     print("Invalid directory...")
    #     all_rep_dir=str(input("Input the all_reports.tsv file directory:\n > "))

    # metadata_dir=str(input("Input the metadata files directory:\n > "))
    # while os.path.exists(metadata_dir)!=True:
    #     print("Invalid directory...")
    #     metadata_dir=str(input("Input the metadata files directory:\n > "))

    all_rep_dir = "C:\\Users\\andre\\OneDrive - FCT NOVA\André\\Mestrado - Bioinfo\\2º Ano\Projeto em Multi-Ómicas - INSA\\teste_1\\testing_files\\testing_merging_and_metadata_files\\barcoded_samples\\all_reports_file_folder"
    metadata_dir = "C:\\Users\\andre\\OneDrive - FCT NOVA\André\\Mestrado - Bioinfo\\2º Ano\Projeto em Multi-Ómicas - INSA\\teste_1\\testing_files\\testing_merging_and_metadata_files\\barcoded_samples\\out_files\\barcode11\\metadata_files"

    os.chdir(all_rep_dir)
    all_rep_name = search_folder(all_rep_dir)
    df = pd.read_csv(all_rep_name[0], sep='\t')
    df1 = merge_all_reports_and_times(df, metadata_dir)

    # print("\nCreating a plot for each category:")
    # for _ in df1.columns:
    #     print("- ", _)
    # param=str(input("> "))
    # while param not in df1.columns:
    #     print(f"{param} not a column option")
    #     param=str(input("> "))

    # print("\nx_axis:\n(must be a column from the all_reports.tsv file)")
    # for _ in df1.columns:
    #     if _!=param:
    #         print("- ", _)
    # x_axis=str(input("> "))
    # while x_axis not in df1.columns:
    #     print(f"{x_axis} not a column option")
    #     x_axis=str(input("> "))

    # print("\ny_axis:\n(must be a column from the all_reports.tsv file)\n")
    # for _ in df1.columns:
    #     if _!=param and _!=x_axis:
    #         print("- ", _)
    # y_axis=str(input("> "))
    # while y_axis not in df1.columns:
    #     print(f"{y_axis} not a column option")
    #     y_axis=str(input("> "))

    # print(f"\nFor each {param} i want to also see the different parameters of:")
    # for _ in df1.columns:
    #     if _!=param and _!=x_axis and _!=y_axis:
    #         print("- ", _)
    # separator=str(input("> "))
    # while separator not in df1.columns:
    #     print(f"{separator} not a column option")
    #     separator=str(input("> "))

    # plot_type=str(input("Plot type:\n('scatter' or 'line')\n > "))
    # while plot_type not in ("line", "scatter"):
    #     plot_type=str(input("Plot type:\n('scatter' or 'line')\n > "))

    # save_or=str(input("Do you wish to save the resultant plots? ('y' or 'n')\n > "))
    # while save_or not in ("y", "n"):
    #     save_or=str(input("Do you wish to save the resultant plots? ('y' or 'n')\n > "))

    save_or = "y"
    # plot_save_format="html"

    if save_or == "y":

        # print("\nSaving plot in which format?")
        # for fo in PLOTTING_OPTIONS:
        #     print("- ", fo)
        # plot_save_format=str(input("> "))
        # while plot_save_format not in PLOTTING_OPTIONS:
        #     print("Invalid format...")
        #     plot_save_format=str(input("> "))

        plot_save_format = "jpeg"
        save_dir = "C:\\Users\\andre\\OneDrive - FCT NOVA\\André\\Mestrado - Bioinfo\\2º Ano\\Projeto em Multi-Ómicas - INSA\\teste_1\\testing_files\\testing_merging_and_metadata_files\\barcoded_samples\\all_reports_file_folder"
        dpi_qual = 600

        # print(f"The plots will be saved in the format of {param}.[{param}].{plot_save_format}\n")
        # save_dir=str(input("\nDirectory to save the resultant plots?\n > "))
        # dpi_qual=int(input("Quality of the image (dpi - higher dpi inscreases file size):\n > "))
    else:
        save_dir = None
        dpi_qual = None

    param = "accID"
    x_axis = "time elapsed"
    y_axis = "Cov (%)"
    plot_type = "line"
    separator = "workflow"

    plot_s_graphics(df1, param, x_axis, y_axis, plot_type,
                    separator, save_or, save_dir, dpi_qual, plot_save_format)


if __name__ == "__main__":
    main()



import os
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import List, Optional, Type

import pandas as pd


class InsafluSampleCodes:
    STATUS_MISSING = 0
    STATUS_UPLOADING = 1
    STATUS_UPLOADED = 2
    STATUS_SUBMITTED = 3
    STATUS_TELEVIR_SUBMITTED = 4
    STATUS_PROCESSING = 5
    STATUS_PROCESSED = 6
    STATUS_SUBMISSION_ERROR = 7
    STATUS_ERROR = 8


@dataclass(frozen=False)
class InsafluFile:
    sample_id: str
    barcode: str
    file_path: str
    remote_path: str
    status: int

    def __post_init__(self):
        self.sample_id = self.sample_id.strip()
        self.barcode = self.barcode.strip()
        self.file_path = self.file_path.strip()
        self.remote_path = self.remote_path.strip()
        self.status = int(self.status)

    def __str__(self):
        return f"{self.sample_id},{self.barcode},{self.file_path},{self.remote_path},{self.status}"

    def __repr__(self):
        return f"{self.sample_id},{self.barcode},{self.file_path},{self.remote_path},{self.status}"

    def __eq__(self, other):
        return self.sample_id == other.sample_id and \
            self.barcode == other.barcode and \
            self.file_path == other.file_path and \
            self.remote_path == other.remote_path and \
            self.status == other.status

    def __hash__(self):
        return hash((self.sample_id, self.barcode, self.file_path, self.remote_path, self.status))


class MetadataEntry():
    """
    MetadataEntry class
    """

    sample_name: str
    fastq1: str
    fastq2: str
    data_set: str
    vaccine_status: str
    week: str
    onset_date: str
    collection_date: str
    lab_reception_date: str
    latitude: str
    longitude: str
    region: str
    country: str
    division: str
    location: str
    time_elapsed: float
    dir: str
    tag: str
    r1_local: str
    r2_local: str

    def __init__(self, sample_name: str, fastq1: str, fastq2: str = "", data_set: str = "", vaccine_status: str = "", week: str = "", onset_date: str = "", collection_date: str = "", lab_reception_date: str = "",
                 latitude: str = "", longitude: str = "", region: str = "", country: str = "", division: str = "", location: str = "", time_elapsed: float = 0, fdir: str = "", tag: str = ""):
        self.sample_name = sample_name
        self.fastq1 = os.path.basename(fastq1)
        self.fastq2 = os.path.basename(fastq2)
        self.data_set = data_set
        self.vaccine_status = vaccine_status
        self.week = week
        self.onset_date = onset_date
        self.collection_date = collection_date
        self.lab_reception_date = lab_reception_date
        self.latitude = latitude
        self.longitude = longitude
        self.region = region
        self.country = country
        self.division = division
        self.location = location
        self.time_elapsed = time_elapsed
        self.dir = fdir
        self.tag = tag
        self.r1_local = fastq1
        self.r2_local = fastq2

    def export_as_dataframe(self):

        metad = pd.DataFrame([self.__dict__])

        metad = metad.rename(columns={
            "sample_name": "sample name",
            "vaccine_status": "vaccine status",
            "onset_date": "onset date",
            "collection_date": "collection date",
            "data_set": "data set",
            "lab_reception_date": "lab reception date",
        })

        return metad



import configparser
import logging
import os
import sys
from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import List, Optional

import pandas as pd

from findontime.connectors import Connector
from findontime.records import InsafluFile, InsafluSampleCodes
from findontime.tables_post import InsafluFilesTable


class UploadStrategy(ABC):
    """
    abstract class to select files"""

    @staticmethod
    @abstractmethod
    def is_to_upload(sample_list: list, sample_index: int) -> bool:
        pass


class UploadAll(UploadStrategy):
    """
    select all files"""

    @staticmethod
    def is_to_upload(sample_list: list, sample_index: int) -> bool:
        return True


class UploadLast(UploadStrategy):
    """
    select last files"""
    @staticmethod
    def is_to_upload(sample_list: list, sample_index: int) -> bool:
        return sample_index == len(sample_list) - 1


class UploadNone(UploadStrategy):
    """
    select no files"""

    @staticmethod
    def is_to_upload(sample_list: list, sample_index: int) -> bool:
        return False


class UploadStep(UploadStrategy):
    """
    select files by step"""

    def __init__(self, step: int):
        self.step = step

    def is_to_upload(self, sample_list: list, sample_index: int) -> bool:
        return sample_index % self.step == 0


class UploadLog:

    STATUS_MISSING = InsafluSampleCodes.STATUS_MISSING
    STATUS_UPLOADING = InsafluSampleCodes.STATUS_UPLOADING
    STATUS_UPLOADED = InsafluSampleCodes.STATUS_UPLOADED
    STATUS_SUBMITTED = InsafluSampleCodes.STATUS_SUBMITTED
    STATUS_PROCESSING = InsafluSampleCodes.STATUS_PROCESSING
    STATUS_PROCESSED = InsafluSampleCodes.STATUS_PROCESSED
    STATUS_SUBMISSION_ERROR = InsafluSampleCodes.STATUS_SUBMISSION_ERROR
    STATUS_ERROR = InsafluSampleCodes.STATUS_ERROR

    columns = [
        "sample_id",
        "barcode",
        "file_path",
        "remote_path",
        "status",
        "tag",
    ]

    def __init__(self) -> None:
        self.log = pd.DataFrame(
            columns=self.columns
        )

    @property
    def available_samples(self) -> List[str]:
        """
        get available samples"""

        return self.log["sample_id"].unique().tolist()

    def get_sample_status_set(self, sample_id: str) -> List[int]:
        """
        get sample status set"""

        return self.log[self.log["sample_id"] == sample_id]["status"].unique().tolist()

    def generate_InsafluFile(self, row: pd.Series) -> InsafluFile:
        """
        generate InsafluSample"""

        return InsafluFile(
            sample_id=row["sample_id"],
            barcode=row["barcode"],
            file_path=row["file_path"],
            remote_path=row["remote_path"],
            status=row["status"]
        )

    def get_sample_files(self, sample_id: str) -> List[InsafluFile]:
        """
        get sample"""

        return [
            self.generate_InsafluFile(row) for _, row in self.log[(self.log["sample_id"] == sample_id) & (self.log["tag"] == "fastq")].iterrows()
        ]

    def generate_fastq_list(self) -> List[InsafluFile]:
        """
        generate samples list"""

        return [
            self.generate_InsafluFile(row) for _, row in self.log[(self.log["tag"] == "fastq")].iterrows()
        ]

    def generate_fastq_list_status(self, status: int) -> List[InsafluFile]:
        """
        generate samples list"""

        return [
            self.generate_InsafluFile(row) for _, row in self.log[(self.log["tag"] == "fastq") & (self.log["status"] == status)].iterrows()
        ]

    def generate_file_list_status(self, status: int) -> List[InsafluFile]:
        """
        generate samples list"""

        return [
            self.generate_InsafluFile(row) for _, row in self.log[(self.log["status"] == status)].iterrows()
        ]

    def get_sample_remotepaths(self, sample_id: str) -> List[str]:
        """
        get sample files"""

        samples = [self.generate_InsafluFile(
            row) for _, row in self.log[self.log["sample_id"] == sample_id].iterrows()]
        sample_files = [sample.remote_path for sample in samples]

        return sample_files

    def check_entry_exists(self, file_path: str) -> bool:
        """
        check entry exists"""

        return file_path in self.log["file_path"].values

    def modify_entry_status(self, file_path: str, status: int) -> None:
        """
        modify entry"""

        self.log.loc[self.log["file_path"] == file_path, "status"] = status

    def update_log(self, sample_id: str, barcode: str, file_path: str, remote_path: str, status: int, tag: Optional[str] = None):
        """
        update upload log"""

        if self.check_entry_exists(file_path):
            self.modify_entry_status(file_path, status)
        else:
            self.new_entry(sample_id, barcode, file_path,
                           remote_path, status, tag)

    def new_entry(self, sample_id: str, barcode: str, file_path: str, remote_path: str, status: int = STATUS_UPLOADED, tag: Optional[str] = None) -> None:
        """
        new entry"""

        if self.check_entry_exists(file_path):
            return

        self.log = pd.concat(
            [
                self.log,
                pd.DataFrame(
                    {
                        "sample_id": [sample_id],
                        "barcode": [barcode],
                        "file_path": [file_path],
                        "remote_path": [remote_path],
                        "status": [status],
                        "tag": [tag]
                    }
                )
            ]
        )

    def get_log(self) -> pd.DataFrame:
        return self.log

    def get_file_status(self, file_path: str) -> int:
        """
        get file status"""

        files = self.log.loc[self.log["file_path"]
                             == file_path, "status"].values

        if len(files) == 0:
            return self.STATUS_MISSING

        return files[0]

    def save_entries_to_db(self, table: InsafluFilesTable) -> None:
        """
        save entries to db"""

        for _, row in self.log.iterrows():
            influ_file = self.generate_InsafluFile(row)
            table.add_sample(influ_file)


class InsafluUpload(ABC):

    """
    requires:
    metadata data frame with merged file path.
    copy method. (copy, move)
    user name.
    """

    logger: UploadLog
    conn: Connector
    TAG_FASTQ = "fastq"
    TAG_METADATA = "metadata"
    remote_dir: str = "/usr/local/web_site/INSaFLU/media/uploads/"
    app_dir = "/usr/local/web_site/INSaFLU/"

    @abstractmethod
    def prep_upload(self):
        """
        prepare upload"""
        pass

    @abstractmethod
    def test_connection(self):
        """
        test connection"""
        pass

    @abstractmethod
    def get_remote_path(self, file_path):
        """
        get remote path"""
        pass

    @abstractmethod
    def check_file_exists(self, file_path):
        """
        check file exists"""
        pass

    @abstractmethod
    def upload_file(self, file_path, remote_path, sample_id: str, barcode: str, tag: str):
        """
        upload file"""
        pass

    @abstractmethod
    def download_file(self, remote_path, local_path):
        """
        download file"""
        pass

    @abstractmethod
    def register_sample(self, fastq_path: str, sample_id: str = "NA", barcode: str = ""):
        """
        register sample using metadir and fastq path"""
        pass

    @abstractmethod
    def upload_sample(self, fastq_path: str, sample_id: str = "NA", barcode: str = ""):
        """
        upload sample using metadir and fastq path"""
        pass

    @abstractmethod
    def update_sample_status_remote(self,  sample_name: str, file_path: str, test=False):
        """
        update sample status"""
        pass

    @abstractmethod
    def submit_sample(self, metadata_path: str):
        """
        submit sample"""
        pass

    @abstractmethod
    def submit_sample_update(self, metadata_path: str, test: bool = False):
        """
        submit sample"""
        pass

    @staticmethod
    def check_submission_success(submission_output) -> bool:
        """
        check submission success"""

        if "Error" in submission_output:
            return False

        if "exists in database" in submission_output:
            return True

        elif "file was processed" in submission_output:
            return True

        return False

    @abstractmethod
    def update_log(self, sample_id: str, barcode: str, file_path: str, remote_path: str, status: int):
        """
        update log"""
        pass

    @staticmethod
    def translate_sample_status(status_output: str):
        """
        translate sample status"""

        if "does not exist" in status_output:
            return InsafluSampleCodes.STATUS_MISSING
        elif "Is Ready" in status_output:
            if "True" in status_output:
                return InsafluSampleCodes.STATUS_SUBMITTED
            else:
                return InsafluSampleCodes.STATUS_UPLOADING

        if "Session terminated" in status_output:
            print("Session terminated")
            sys.exit(1)

        return InsafluSampleCodes.STATUS_ERROR

    @staticmethod
    def translate_televir_submission_output(submission_output):
        """
        translate televir submission output"""

        if "submitted" in submission_output:
            if "already" in submission_output:
                return InsafluSampleCodes.STATUS_TELEVIR_SUBMITTED

            return InsafluSampleCodes.STATUS_TELEVIR_SUBMITTED

        return InsafluSampleCodes.STATUS_SUBMISSION_ERROR

    @staticmethod
    def translate_televir_status_output(status_output):

        if "Error" in status_output:
            return InsafluSampleCodes.STATUS_ERROR

        return InsafluSampleCodes.STATUS_PROCESSING

    @abstractmethod
    def get_sample_status(self, sample_name, test_ready: bool = False, test_missing: bool = False) -> int:
        """
        get sample status"""
        pass

    @abstractmethod
    def clean_upload(self, file_path: str):
        """
        clean upload"""
        pass

    @abstractmethod
    def launch_televir_project(self, sample_name: str, project_name: Optional[str] = None):
        """
        launch televir project"""
        pass

    @abstractmethod
    def deploy_televir_sample(self, sample_id: str, sample_name: str, file_path: str, project_name: Optional[str] = None, test: bool = False):
        """
        deploy televir sample"""
        pass

    @abstractmethod
    def translate_project_results(self,  project_results: str):
        """
        get project id"""
        pass

    @abstractmethod
    def get_project_results(self, project_name: str, output_dir: str, test: bool = False):
        """
        get project results"""
        pass


class InsafluUploadRemote(InsafluUpload):

    # conn: Connector
    logger: UploadLog
    TAG_FASTQ = "fastq"
    TAG_METADATA = "metadata"

    def __init__(self, connector: Connector, config_file: str) -> None:
        super().__init__()
        self.logger = UploadLog()
        self.conn = connector

        self.logging_logger = logging.getLogger("insaflu_upload")
        self.logging_logger.setLevel(logging.DEBUG)

        self.prep_config(config_file=config_file)
        self.prep_upload()
        self.test_insaflu_user_exists()

    def input_config(self, config_file: str):
        """
        input config"""

        config = configparser.ConfigParser()
        config.read(config_file)

        username = config["INSAFLU"].get("username", None)
        app_dir = config["INSAFLU"].get("app_dir", None)

        if username is None:
            raise ValueError("username not found in config file")

        if app_dir is None:
            raise ValueError("app_dir not found in config file")

        return username, app_dir

    def input_user(self):
        """
        input user"""

        username = input("INSaFLU username: ")
        app_dir = input("INSaFLU application directory: ")

        return username, app_dir

    def prep_config(self, config_file: Optional[str] = None):
        """
        prepare user"""

        if config_file is None:
            self.televir_user, self.app_dir = self.input_user()
        else:
            try:
                self.televir_user, self.app_dir = self.input_config(
                    config_file=config_file)
            except ValueError:
                self.televir_user, self.app_dir = self.input_user()

    def prep_upload(self):
        """
        prepare upload"""
        self.django_manager = os.path.join(
            self.app_dir,
            "manage.py"
        )

        self.remote_dir = os.path.join(
            self.app_dir,
            "media",
            "uploads"
        )

        self.test_connection()

    def test_connection(self):
        """
        test connection
        """

        self.conn.test_connection()

    def test_insaflu_user_exists(self):
        """
        test insaflu user exists"""

        command = "python3 {} show_all_users".format(self.django_manager)

        output = self.conn.execute_command(command=command)

        if self.televir_user not in output:
            self.logging_logger.error(
                f"Insaflu user {self.televir_user} does not exist")
            sys.exit(1)

    def update_log(self, sample_id: str, barcode: str, file_path: str, remote_path: str, status: int = UploadLog.STATUS_UPLOADED):
        """
        update upload log"""

        self.logger.update_log(
            sample_id=sample_id,
            barcode=barcode,
            file_path=file_path,
            remote_path=remote_path,
            status=status
        )

    def update_file_status(self, file_path: str, status: int):

        self.logger.update_log(
            sample_id="NA",
            barcode="NA",
            file_path=file_path,
            remote_path="NA",
            status=status
        )

    def get_remote_path(self, file_path):

        return os.path.join(
            self.remote_dir,
            os.path.basename(file_path)
        )

    def check_file_exists(self, file_path: str):
        """
        check file exists"""

        return self.conn.check_file_exists(file_path)

    def upload_file(self, file_path: str, remote_path: str, sample_id="NA", barcode="", tag: Optional[str] = None):
        """
        upload file to remote server"""

        status = self.logger.STATUS_MISSING
        if self.conn.check_file_exists(remote_path):
            status = self.logger.STATUS_UPLOADED
            self.logging_logger.error(f"File already exists: {file_path}")

        else:
            try:
                self.conn.upload_file(
                    file_path,
                    remote_path)

                status = self.logger.STATUS_UPLOADED

            except Exception as error:

                self.logging_logger.error(f"Error uploading file: {file_path}")
                self.logging_logger.error(error)
                status = self.logger.STATUS_ERROR

        self.logger.update_log(
            sample_id=sample_id,
            barcode=barcode,
            file_path=file_path,
            remote_path=remote_path,
            status=status,
            tag=tag
        )

    def download_file(self, remote_path: str, local_path: str):
        """
        download file from remote server"""

        if self.conn.check_file_exists(remote_path):
            try:
                self.conn.download_file(remote_path, local_path)
                self.logging_logger.info(f"File downloaded: {remote_path}")

            except Exception as error:
                self.logging_logger.info(
                    f"Error downloading file: {remote_path}")
                self.logging_logger.info(error)

    def register_sample(self, fastq_path: str, sample_id: str = "NA", barcode: str = ""):

        self.logger.new_entry(
            sample_id=sample_id,
            barcode=barcode,
            file_path=fastq_path,
            remote_path=self.get_remote_path(fastq_path),
            status=InsafluSampleCodes.STATUS_MISSING,
            tag=self.TAG_FASTQ
        )

    def upload_sample(self, fastq_path: str, sample_id: str = "NA", barcode: str = ""):
        """
        upload sample using metadir and fastq path"""

        self.upload_file(
            fastq_path,
            self.get_remote_path(fastq_path),
            sample_id,
            barcode,
            self.TAG_FASTQ
        )

    def update_sample_status_remote(self, sample_name: str, file_path: str, test: bool = False):
        """
        update sample status"""
        status = self.get_sample_status(
            sample_name, test_ready=test)

        self.update_file_status(
            file_path,
            status,
        )

        return status

    def submit_sample(self, remote_metadata_path: str, test: bool = False):
        """
        submit sample metadata"""

        bash_command = f"python3 {self.django_manager} upload_samples --metadata_file {remote_metadata_path} --user_login {self.televir_user}"

        if test:
            bash_command += " --test"

        output = self.conn.execute_command(
            bash_command
        )

        return output

    def submit_sample_update(self, metadata_path: str, test: bool = False):
        """
        submit sample"""
        remote_metadata_path = self.get_remote_path(metadata_path)

        if not self.check_file_exists(remote_metadata_path):
            self.logging_logger.info(
                f"Remote metadata file does not exist: {remote_metadata_path}")
            return

        output = self.submit_sample(remote_metadata_path, test=test)

        success = self.check_submission_success(output)

        if success:
            self.logging_logger.info(
                f"Metadata submission success: {metadata_path}")

        success_tag = InsafluSampleCodes.STATUS_SUBMITTED if success else InsafluSampleCodes.STATUS_SUBMISSION_ERROR

        self.update_file_status(
            file_path=metadata_path,
            status=success_tag
        )

    def get_sample_status(self, sample_name: str, test_ready: bool = False, test_missing: bool = False):
        """
        get sample status"""

        command = f"python3 {self.django_manager} check_sample_status --name {sample_name} --user_login {self.televir_user}"

        if test_ready:
            command += " --test_ready"

        if test_missing:
            command += " --test_missing"

        sample_status = self.conn.execute_command(
            command
        )

        return self.translate_sample_status(sample_status)

    def clean_upload(self, file_path: str):
        """
        clean upload"""
        file_exists = self.check_file_exists(file_path)

        if file_exists:
            _ = self.conn.execute_command(
                f"rm -f {file_path}"
            )

    def launch_televir_project(self, sample_name: str, project_name: Optional[str] = None, test: bool = False):
        """
        launch televir project"""

        if project_name is None:
            project_name = sample_name

        command = [
            "python3",
            self.django_manager,
            "create_televir_from_sample",
            "--sample_name",
            sample_name,
            "--user_login",
            self.televir_user,
            "--project_name",
            project_name
        ]

        if test:
            command.append("--test")

        command = " ".join(command)

        submit_status = self.conn.execute_command(
            command
        )

        return self.translate_televir_submission_output(submit_status)

    def rm_sample_files_remote(self, sample_id: str):

        sample_files = self.logger.get_sample_remotepaths(sample_id)

        for file in sample_files:
            self.clean_upload(
                file,
            )

    def deploy_televir_sample(self, sample_id: str, sample_name: str, file_path: str, project_name: Optional[str] = None, test: bool = False):
        """
        deploy televir sample"""
        submit_status = self.launch_televir_project(
            sample_name, project_name, test=test
        )

        if submit_status == InsafluSampleCodes.STATUS_TELEVIR_SUBMITTED:

            self.rm_sample_files_remote(
                sample_id,
            )

            self.update_file_status(
                file_path,
                submit_status,
            )

        return submit_status

    def translate_project_results(self, project_results: str) -> str:
        project_file = project_results.splitlines()[-1]

        if not project_file.startswith("/"):
            return ""

        return project_file

    def get_project_results(self, project_name: str, local_file: str, test: bool = False):

        command = [
            "python3",
            self.django_manager,
            "check_televir_results",
            "--user_login",
            self.televir_user,
            "--project_name",
            project_name
        ]

        if test:
            command.append("--test")

        command = " ".join(command)

        submit_status = self.conn.execute_command(
            command
        )

        project_file = self.translate_project_results(submit_status)

        if project_file:

            self.download_file(
                project_file, local_file
            )




import datetime
import logging
import os
import sys
from typing import List

import pandas as pd
from fastq_handler.fastq_handler import DirectoryProcessingSimple, PreMain
from fastq_handler.records import Processed

from findontime.configs import InfluConfig, default_log_handler

from findontime.plot_utils import plot_project_results
from findontime.records import InsafluFile, MetadataEntry
from findontime.upload_utils import InsafluSampleCodes, InsafluUpload


class InfluProcessed(Processed):
    """
    TelevirProcessed class
    """

    def __init__(self, output_dir: str):
        super().__init__(output_dir)

    def generate_metadata_entry_row(self, row: pd.Series):
        """
        create tsv
        """
        fastq_dir = row.dir
        fastq_file = row.fastq
        merged_file = row.merged

        new_entry = self.generate_metadata_entry(
            fastq_file, fastq_dir, merged_file)

        return new_entry.export_as_dataframe()
    

    def generate_metadata_entry(self, fastq_file: str, fastq_dir: str, merged_file: str, tag=""):

        sample_name, _ = self.get_run_barcode(
            merged_file, fastq_dir)

        time_elapsed = self.get_file_time(
            fastq_file=fastq_file,
            fastq_dir=fastq_dir
        )
        new_entry = MetadataEntry(
            sample_name=sample_name,
            fastq1=merged_file,
            time_elapsed=float(time_elapsed),
            fdir=os.path.dirname(merged_file),
            tag=tag
        )

        return new_entry

    def generate_metadata(self):
        """
        create tsv
        """
        metadata = pd.DataFrame()

        for index, row in self.processed.iterrows():
            metadata = pd.concat(
                [metadata, self.generate_metadata_entry_row(row)])

        return metadata

    def export_metadata(self, run_metadata: InfluConfig):
        """
        export metadata
        """
        influ_metadata = self.generate_metadata()

        influ_metadata.to_csv(
            os.path.join(
                run_metadata.metadata_dir,
                run_metadata.tsv_temp_name
            ),
            sep="\t",
            index=False,
        )


class InfluDirectoryProcessing(DirectoryProcessingSimple):
    """
    TelevirDirectoryProcessing class
    replace directory gen to include metadata
    """

    metadata_suffix: str = "_metadata.tsv"

    uploader: InsafluUpload

    def __init__(self, fastq_dir: str, run_metadata: InfluConfig, processed: InfluProcessed,
                 start_time: float, test: bool = False):
        super().__init__(fastq_dir, run_metadata, processed, start_time)

        self.run_metadata = run_metadata
        self.processed = processed
        self.uploader = run_metadata.uploader
        self.test = test

    def prep_output_dirs(self):
        """create output dirs"""

        for outdir in [
            self.merged_gz_dir,
        ]:
            os.makedirs(outdir, exist_ok=True)

    @staticmethod
    def get_filename_from_path(file_name):
        filename = os.path.basename(file_name)
        filename, ext = os.path.splitext(filename)
        if ext == ".gz":
            filename, ext = os.path.splitext(filename)
        return filename

    def metadata_name_for_sample(self, sample_name):
        filename = self.get_filename_from_path(sample_name)

        return filename + self.metadata_suffix

    def register_sample(self, metadata_entry: MetadataEntry):
        """
        register sample to remote server"""

        if self.uploader.check_file_exists(metadata_entry.r1_local):
            return

        _, barcode = self.processed.get_run_barcode(
            metadata_entry.fastq1, self.fastq_dir)

        sample_id = self.processed.get_sample_id_from_merged(
            metadata_entry.fastq1
        )

        self.uploader.register_sample(
            metadata_entry.r1_local,
            sample_id=sample_id,
            barcode=barcode,
        )

    def upload_sample(self, metadata_entry: MetadataEntry):
        """
        upload sample to remote server"""

        _, barcode = self.processed.get_run_barcode(
            metadata_entry.fastq1, self.fastq_dir)

        sample_id = self.processed.get_sample_id_from_merged(
            metadata_entry.fastq1
        )

        self.uploader.upload_sample(
            metadata_entry.r1_local,
            sample_id=sample_id,
            barcode=barcode,
        )

    def insaflu_process(self):
        """
        prepare processed files for upload
        """

        files_to_upload = self.processed.processed_fastq_list()

        processed_df = self.processed.processed_entries

        for ix, row in processed_df.iterrows():

            fastq_file = row.fastq
            merged_file = row.merged
            merged_name = self.get_filename_from_path(merged_file)

            if self.run_metadata.upload_strategy.is_to_upload(files_to_upload, ix):
                status = self.uploader.get_sample_status(
                    merged_name, test_missing=self.test)

                metadata_entry = self.processed.generate_metadata_entry(
                    fastq_file, self.fastq_dir, merged_file, tag=self.run_metadata.name_tag
                )

                self.register_sample(metadata_entry)

                if status in [
                    InsafluSampleCodes.STATUS_MISSING,
                    InsafluSampleCodes.STATUS_ERROR,
                ]:

                    self.upload_sample(
                        metadata_entry
                    )

    def process_folder(self):
        """
        process folder, merge and update metadata
        submit to televir only the last file.
        """
        super().process_folder()
        self.insaflu_process()


class InsafluSetup:
    """
    InsafluSetup class"""

    def __init__(self, run_metadata: InfluConfig):
        self.run_metadata = run_metadata
        self.uploader = run_metadata.uploader
        self.processed = InfluProcessed(
            self.run_metadata.logs_dir,
        )


class PreMainWithMetadata(PreMain):
    """
    """
    metadata_dirname = "metadata_dir"

    processed: InfluProcessed

    def __init__(self, run_metadata: InfluConfig):
        super().__init__(run_metadata)
        self.logger = logging.getLogger(__name__)
        self.logger.addHandler(default_log_handler)

        self.prep_metadata_dir()
        self.run_metadata = run_metadata
        self.processed = InfluProcessed(
            self.run_metadata.logs_dir,
        )

    def prep_metadata_dir(self):
        """
        create output directories
        """

        os.makedirs(self.run_metadata.metadata_dir, exist_ok=True)

    def write_metadata(self, metadata: List[MetadataEntry], metadata_filename: str):
        """
        update metadata
        """

        metadata_df = [
            x.export_as_dataframe() for x in metadata
        ]

        metadata_df = pd.concat(metadata_df)

        metadata_df.to_csv(
            metadata_filename,
            sep="\t",
            index=False,
        )

    def metadata_from_files(self, files_list: List[InsafluFile]) -> List[MetadataEntry]:
        """
        generate metadata from files
        """
        metadata_list = [
            self.processed.generate_metadata_entry(
                x.file_path,
                os.path.dirname(x.file_path),
                x.remote_path,
                tag=self.run_metadata.name_tag
            ) for x in files_list
        ]
        return metadata_list

    def generate_metatadata_filename(self):
        """
        generate metadata filename
        """
        time_now = datetime.datetime.now()
        formatted_time = time_now.strftime("%Y%m%d_%H%M%S")

        metadata_filename = f"{self.run_metadata.name_tag}_{formatted_time}_metadata.tsv"
        return metadata_filename

    def get_samples_to_submit(self):
        """
        get samples to submit
        """
        files_to_upload = self.processed.processed.fastq.tolist()
        merged_files = self.processed.processed.merged.tolist()
        dirs = self.processed.processed.dir.tolist()

        insaflu_file_list = []
        for i, fastq1 in enumerate(files_to_upload):

            _, barcode = self.processed.get_run_barcode(fastq1, self.fastq_dir)
            sample_id = self.processed.get_sample_id_from_merged(
                merged_files[i]
            )

            insaflu_file_list.append(
                InsafluFile(
                    sample_id=sample_id,
                    barcode=barcode,
                    file_path=os.path.join(dirs[i], fastq1),
                    remote_path=merged_files[i],
                    status=0)
            )

        return insaflu_file_list

    def metadata_prepare(self):
        """
        submit sample to remote server"""

        samples_to_submit = self.get_samples_to_submit()
        sample_metadata = self.metadata_from_files(samples_to_submit)

        if len(sample_metadata) == 0:
            return ""

        self.logger.info(f"Submitting {len(sample_metadata)} sample(s)")

        insaflu_metadata_file = self.generate_metatadata_filename()
        metadata_filepath = os.path.join(
            self.run_metadata.metadata_dir,
            insaflu_metadata_file
        )

        self.write_metadata(
            sample_metadata,
            metadata_filepath
        )

        return metadata_filepath

    def run(self):
        """
        run main
        """
        super().run()
        metadata_filepath = self.metadata_prepare()

        return metadata_filepath


class InsafluFileProcess(PreMainWithMetadata, InsafluSetup):
    """
    InsafluUpload class
    """

    processed: InfluProcessed
    run_metadata: InfluConfig
    projects_results: list = []
    metadata_dirname = "metadata_dir"

    def __init__(self, run_metadata: InfluConfig):
        PreMainWithMetadata.__init__(self, run_metadata)
        InsafluSetup.__init__(self, run_metadata)

        self.test = False

    def update_projects(self, project_file: str):
        """
        update project added
        """
        if project_file not in self.projects_results:
            if os.path.exists(project_file):
                self.projects_results.append(project_file)

    def get_directory_processing(self, fastq_dir: str):

        return InfluDirectoryProcessing(fastq_dir, self.run_metadata, self.processed,
                                        self.start_time, test=self.test)

    def get_samples_to_submit(self) -> List[InsafluFile]:
        """
        get samples to submit
        """
        samples_to_submit = self.uploader.logger.generate_fastq_list_status(
            InsafluSampleCodes.STATUS_UPLOADED)

        return samples_to_submit

    def clean_remote(self):
        """
        clean remote server
        """
        samples_to_clean = self.uploader.logger.generate_file_list_status(
            InsafluSampleCodes.STATUS_SUBMITTED)

        for file in samples_to_clean:
            self.uploader.clean_upload(file.remote_path)

    def submit_samples(self, metadata_filepath: str):
        """
        submit samples
        """

        if not os.path.exists(metadata_filepath):
            return

        insaflu_metadata_file = os.path.basename(metadata_filepath)

        self.uploader.upload_file(
            metadata_filepath,
            self.uploader.get_remote_path(insaflu_metadata_file),
            "metad",
            "metad",
            self.uploader.TAG_METADATA,
        )

        status = self.uploader.logger.get_file_status(metadata_filepath)

        if status == InsafluSampleCodes.STATUS_UPLOADED:

            self.uploader.submit_sample_update(
                metadata_filepath, test=self.test
            )

    def export_global_metadata(self):

        self.prep_metadata_dir()
        self.processed.export_metadata(
            self.run_metadata
        )

    def save_to_db(self):
        """
        save to db
        """
        if self.run_metadata.tables.is_connected:
            self.uploader.logger.save_entries_to_db(
                self.run_metadata.tables.insaflu_files)

    def monitor_samples_status(self):
        """
        process samples
        """

        fastq_list = self.uploader.logger.generate_fastq_list()

        for fastq in fastq_list:

            file_name, _ = self.processed.get_run_info(fastq.file_path)

            _ = self.uploader.update_sample_status_remote(
                file_name, fastq.file_path, test=self.test)

    def run(self):
        PreMainWithMetadata.run(self)
        metadata_filepath = self.metadata_prepare()
        self.submit_samples(metadata_filepath)
        self.clean_remote()
        self.monitor_samples_status()
        self.export_global_metadata()
        self.save_to_db()


class TelevirFileProcess(InsafluSetup):
    """
    InsafluUpload class
    """

    processed: InfluProcessed
    run_metadata: InfluConfig
    projects_results: list = []
    output_dirname: str = "televir_results"
    output_dir: str = "metagen"

    def __init__(self, run_metadata: InfluConfig):
        super().__init__(run_metadata)

        self.logger = logging.getLogger(__name__)
        self.logger.addHandler(default_log_handler)

        self.real_sleep = self.run_metadata.sleep_time
        self.prep_output_dir()

        self.test = False

    def prep_output_dir(self):
        """
        create output dir
        """
        if self.run_metadata.deploy_televir:

            self.output_dir = os.path.join(
                self.run_metadata.output_dir,
                self.output_dirname
            )

            os.makedirs(
                self.output_dir, exist_ok=True
            )

    def update_projects(self, project_file: str):
        """
        update project added
        """
        if project_file not in self.projects_results:
            if os.path.exists(project_file):
                self.projects_results.append(project_file)

    def deploy_televir_sample(self, sample_id, file_name, file_path, project_name):
        """
        deploy televir sample
        """

        status = self.uploader.get_sample_status(
            file_name, test_ready=self.test)

        if status == InsafluSampleCodes.STATUS_SUBMITTED:

            status = self.uploader.deploy_televir_sample(
                sample_id,
                file_name,
                file_path,
                project_name,
                test=self.test
            )

    def assign_project_name(self, insaflu_file: InsafluFile):
        """
        assign project name
        """
        project_name = self.run_metadata.name_tag

        return project_name

    def deploy_televir_batch(self):
        """
        deploy televir batch
        """
        self.logger.info("Deploying televir batch")

        fastq_list = self.uploader.logger.generate_fastq_list_status(
            InsafluSampleCodes.STATUS_SUBMITTED)
        
        for fastq in fastq_list:
            
            project_name = self.assign_project_name(fastq)

            file_name, _ = self.processed.get_run_info(fastq.file_path)
            self.deploy_televir_sample(
                fastq.sample_id,
                file_name,
                fastq.file_path,
                project_name,
            )

    def download_project_results(self):
        """
        download project results
        """
        self.logger.info("Downloading project results")

        for sample_id in self.uploader.logger.available_samples:

            if InsafluSampleCodes.STATUS_TELEVIR_SUBMITTED in self.uploader.logger.get_sample_status_set(sample_id):

                project_file = os.path.join(
                    self.output_dir,
                    sample_id + ".tsv"
                )

                self.uploader.get_project_results(
                    sample_id, project_file, test=self.test
                )

                self.update_projects(project_file)

    def run(self):

        if self.run_metadata.deploy_televir:

            self.deploy_televir_batch()

            self.download_project_results()

            self.logger.info("Plotting results")
            _ = plot_project_results(
                self.projects_results, self.processed.processed_entries, self.output_dir)

    def run_return_plot(self):

        if self.run_metadata.deploy_televir:

            self.deploy_televir_batch()

            self.download_project_results()
            return plot_project_results(
                self.projects_results,  self.processed.processed_entries, self.run_metadata.output_dir, write_html=False)

        return None




import signal
import sys
import time

from findontime.manager import MainInsaflu


def signal_handler(signal, frame):
    print("exiting")
    sys.exit(0)


def main():
    signal.signal(signal.SIGINT, signal_handler)
    main_insaflu = MainInsaflu()
    main_insaflu.run()


if __name__ == "__main__":
    main()




import os
import sys
from typing import List

import dash_bootstrap_components as dbc
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import plotly.express as px
import plotly.graph_objects as go
import seaborn as sns
from plotly.subplots import make_subplots


def get_sample_name_from_merged(merged):
    file_name = os.path.basename(merged)
    file_name, ext = os.path.splitext(file_name)
    if ext == ".gz":
        file_name, ext = os.path.splitext(file_name)

    return file_name


def get_sample_runs(sample_name, results_df):
    sample_df = results_df[results_df["sample_name"] == sample_name]
    return sample_df["run_id"].unique()


def accid_per_sample(accid_list, sample_names, results_df):
    """
    return results df with accid entries for each sample for each accid in results"""

    accid_stats = pd.DataFrame()

    for accid in accid_list:
        for sample_name in sample_names:

            sample_df = results_df[results_df["sample_name"] == sample_name]
            time_elapsed = sample_df["time_elapsed"].unique()[0]
            description = sample_df["description"].unique()[0]
            taxid = sample_df["taxid"].unique()[0]
            sample_df = sample_df[sample_df["accid"] == accid]

            if sample_df.shape[0] == 0:
                sample_stat = pd.DataFrame({"coverage": [np.nan], "depth": [np.nan], "ref_proportion": [
                    np.nan], "taxid": [taxid], "mapped_reads": [np.nan],
                    "description": [description], "time_elapsed": [time_elapsed]})
            else:
                sample_stat = sample_df[
                    [
                        "coverage",
                        "depth",
                        "ref_proportion",
                        "description",
                        "taxid",
                        "mapped_reads",
                        "time_elapsed",
                        "leaf_id"]
                ]

            sample_stat.insert(0, "accid", accid)
            sample_stat.insert(0, "sample_name", sample_name)

            accid_stats = pd.concat([accid_stats, sample_stat])

    return accid_stats


# metadir = "/home/bioinf/Desktop/CODE/INSaFLU-TELEVIR_cml_upload/test_new"
# processed_file = os.path.join(metadir, "processed.tsv")
#
# results_dir = "/home/bioinf/Desktop/CODE/INSaFLU-TELEVIR_cml_upload/test_new"
# results_file = os.path.join(results_dir, "barcode_01_another.tsv")
#
# plots_dir = "/home/bioinf/Desktop/CODE/INSaFLU-TELEVIR_cml_upload/test_new/plots"


class TelevirPlotResults:
    stats: List[str] = ["coverage", "depth", "ref_proportion", "mapped_reads"]

    def __init__(self, results_df: pd.DataFrame, processed_df: pd.DataFrame):
        self.results_df = results_df
        self.processed_df = processed_df

        self.prep_accid_stats()

    def preprocess_df(self):
        self.processed_df["sample_name"] = self.processed_df["merged"].apply(
            get_sample_name_from_merged)

        self.results_df["time_elapsed"] = self.results_df["sample_name"].map(
            self.processed_df.set_index("sample_name")["time"])

    def prep_accid_stats(self):
        self.preprocess_df()

        sample_names = self.results_df["sample_name"].unique()
        accids = self.results_df["accid"].unique()

        self.accid_stats = accid_per_sample(
            accids, sample_names, self.results_df)

    def plot_accid_stats_max(self, stat: str = "coverage", ax=None):
        """
        Plots the max value of a stat for every accid by time elapsed.
        """
        data_max = self.accid_stats.groupby(["accid", "time_elapsed"])[
            stat].max().reset_index()

        fig = px.line(data_max, x="time_elapsed", y=stat,
                      color="accid", template="seaborn")

        return fig


def plot_project_results(projects_results: List[str], processed: pd.DataFrame, plots_dir: str, write_html: bool = True):
    """
    plot project results
    """

    stat = "coverage"

    fig = go.Figure()

    figures = []

    for project_result_df in projects_results:

        try:
            project_result_df = pd.read_csv(project_result_df, sep="\t")
        except pd.errors.EmptyDataError:
            continue

        plotter = TelevirPlotResults(
            project_result_df,
            processed,
        )

        fig_proj = plotter.plot_accid_stats_max(stat=stat)

        figures.append(fig_proj)

    if len(figures) == 0:
        return None

    fig = make_subplots(rows=len(figures), cols=1)

    for i, figure in enumerate(figures):
        for trace in range(len(figure["data"])):
            fig.append_trace(figure["data"][trace], row=i+1, col=1)

    if write_html:

        fig.write_html(os.path.join(plots_dir, "project_results.html"))
        return None

    else:
        return fig



import os
import signal
import sys
import threading
import time
from _thread import interrupt_main
from threading import Event, Thread

from findontime.insaflu_uploads import InsafluFileProcess, TelevirFileProcess


class LockWithOwner:

    lock = threading.RLock()
    owner = 'A'

    def acquire_for(self, owner):
        n = 0
        while True:
            self.lock.acquire()
            if self.owner == owner:
                break
            n += 1
            self.lock.release()

    def release_to(self, new_owner):
        self.owner = new_owner
        self.lock.release()


class InsafluFileProcessThread(Thread):
    def __init__(self, compressor: InsafluFileProcess, thread_lock: LockWithOwner):
        super(InsafluFileProcessThread, self).__init__()
        self.lock = thread_lock
        self.processor = compressor
        self._stopevent = Event()  # initialize the event
        self.counter = 0
        self.error = False
        self.schedule_stop = False

    def run(self):
        try:
            while not self._stopevent.is_set():
                self._stopevent.clear()  # Make sure the thread is unset
                self.lock.acquire_for("A")

                print("--------------------")
                print("Processing files, cycle number: {}".format(
                    self.counter))
                self.processor.run()

                self.counter += 1

                if self.counter == 1:
                    if self.processor.run_metadata.monitor is False:
                        self._stopevent.set()

                self.lock.release_to('B')

        except KeyboardInterrupt:
            pass

        except Exception as e:
            print("Error in thread, stopping...")
            print(e)
            self.error = True
            self.stop()
            interrupt_main()

    def stop(self):
        self._stopevent.set()

    def join(self, timeout=None):
        self._stopevent.set()
        Thread.join(self, timeout)


class TelevirFileProcessThread(Thread):
    def __init__(self, processor: TelevirFileProcess, thread_lock: LockWithOwner):
        super(TelevirFileProcessThread, self).__init__()
        self.lock = thread_lock
        self.processor = processor
        self._stopevent = Event()  # initialize the event
        self.work_period = processor.real_sleep
        self.counter = 0
        self.error = False

    def run(self):
        try:

            while not self._stopevent.is_set():
                start_time = time.time()
                execution_time = 0
                self._stopevent.clear()  # Make sure the thread is unset
                self.lock.acquire_for("B")

                while execution_time < self.work_period:
                    self.processor.run()
                    execution_time = time.time() - start_time

                    self.counter += 1

                    if self.counter == 1:
                        if self.processor.run_metadata.monitor is False:
                            self._stopevent.set()
                            break

                    time.sleep(1)

                self.lock.release_to('A')

        except KeyboardInterrupt:
            raise KeyboardInterrupt

        except Exception as e:
            print("Error in TelevirFileProcessThread")
            print(e)
            self.error = True

            self.stop()

            interrupt_main()

    def stop(self):
        self._stopevent.set()

    def join(self, timeout=None):
        self._stopevent.set()
        Thread.join(self, timeout)


def signal_handler(signal, frame):
    print("Exiting program")
    sys.exit(0)



import argparse
import os
import signal
import sys
import time
from dataclasses import dataclass
from typing import Protocol, Tuple

from fastq_handler.actions import ProcessActionDownsize, ProcessActionMergeWithLast

from findontime.configs import InfluConfig, RunConfigMeta
from findontime.connectors import ConnectorDocker, ConnectorParamiko
from findontime.drones import (
    InsafluFileProcessThread,
    LockWithOwner,
    TelevirFileProcessThread,
    signal_handler,
)
from findontime.insaflu_uploads import (
    InsafluFileProcess,
    PreMainWithMetadata,
    TelevirFileProcess,
)
from findontime.upload_utils import (
    InsafluUploadRemote,
    UploadAll,
    UploadLast,
    UploadNone,
)


def get_arguments():
    parser = argparse.ArgumentParser(description="Process fastq files.")
    parser.add_argument("-i", "--in_dir", help="Input directory", required=True)
    parser.add_argument("-o", "--out_dir", help="Output directory", required=True)
    parser.add_argument(
        "-s",
        "--sleep",
        help="Sleep time between checks in monitor mode",
        default=600,
        type=int,
    )

    parser.add_argument(
        "-n",
        "--tag",
        help="name tag, if given, will be added to the output file names",
        required=False,
        type=str,
        default="some_tag",
    )

    parser.add_argument(
        "--config", help="config file", required=False, type=str, default="config.ini"
    )

    parser.add_argument(
        "--max_size",
        help="max size of the output file, in kilobytes",
        type=int,
        default=400000,
    )

    parser.add_argument("--merge", help="merge files", action="store_true")

    parser.add_argument("--downsize", help="downsize fastq files", action="store_true")

    parser.add_argument(
        "--upload",
        default="last",
        choices=["last", "all", "none"],
        help="file upload stategy (default: last)",
    )

    parser.add_argument(
        "--connect",
        default="docker",
        choices=["docker", "ssh"],
        help="file upload stategy (default: docker)",
    )

    parser.add_argument(
        "--keep_names", help="keep original file names", action="store_true"
    )

    parser.add_argument(
        "--monitor", help="monitor directory until killed", action="store_true"
    )

    parser.add_argument(
        "--televir",
        help="deploy televir pathogen identification on each sample",
        action="store_true",
    )

    return ArgsClass(**parser.parse_args().__dict__)


@dataclass
class ArgsClass:
    in_dir: str
    out_dir: str
    sleep: int
    tag: str
    config: str
    merge: bool
    downsize: bool
    upload: str
    connect: str
    keep_names: bool
    monitor: bool
    televir: bool
    max_size: int

    def __post_init__(self):
        self.in_dir = os.path.abspath(self.in_dir)
        self.out_dir = os.path.abspath(self.out_dir)
        self.config = os.path.abspath(self.config)
        self.max_size = self.max_size * 1000


class MainInsaflu:
    def __init__(self):
        pass

    def select_manager(self, args: ArgsClass):
        if args.upload == "none":
            print("No upload specified, using fastq handler")
            return FastqHandlerManager(args)
        else:
            return InsafluManager(args)

    def run(self):
        args = get_arguments()

        if args.merge is True and args.upload == "all":
            ### inform user of behaviour and ask if to continue
            print(
                "Merge and upload all will result in successive files being merged and uploaded, is this what you want?"
            )
            print("If not, please use --upload last or --upload none")
            print("If you want to continue, please press enter")
            user_input = input()
            if user_input != "":
                sys.exit(0)

        manager = self.select_manager(args)
        manager.run()


class Deployer(Protocol):
    def __init__(self, args: ArgsClass) -> None:
        ...

    def run(self) -> None:
        ...


class FastqHandlerManager:
    def __init__(self, args: ArgsClass):
        self.args = args

    def setup_config(self, args: ArgsClass):
        actions = []

        if args.merge:
            actions.append(ProcessActionMergeWithLast())

        if args.downsize:
            actions.append(ProcessActionDownsize(args.max_size))

        if not actions:
            print("No actions specified, will merge files by default")
            actions.append(ProcessActionMergeWithLast())

        run_metadata = RunConfigMeta(
            fastq_dir=args.in_dir,
            output_dir=args.out_dir,
            name_tag=args.tag,
            actions=actions,
            keep_name=args.keep_names,
            sleep_time=args.sleep,
            max_size=(args.max_size),
        )
        actions = []

        return run_metadata

    def setup_compressor(self, run_metadata: RunConfigMeta):
        compressor = PreMainWithMetadata(run_metadata)

        return compressor

    def run(self):
        compressor = self.setup_compressor(self.setup_config(self.args))

        if self.args.monitor:
            compressor.run_until_killed()

        else:
            compressor.run()


class InsafluManager:
    def __init__(self, args: ArgsClass):
        self.args = args

    def setup_config(self, args: ArgsClass, test=False):
        # check input dir exists
        if not os.path.isdir(args.in_dir):
            print("Input directory does not exist")
            sys.exit(1)

        # check config file exists
        if not os.path.isfile(args.config):
            print("Config file does not exist")
            sys.exit(1)

        # create connector
        if args.connect == "docker":
            connector = ConnectorDocker(args.config)
            if test:
                connector.set_interactive(False)

        else:
            connector = ConnectorParamiko(args.config)

        insaflu_upload = InsafluUploadRemote(connector, args.config)

        # determine upload strategy
        if args.upload == "last":
            upload_strategy = UploadLast
        elif args.upload == "none":
            upload_strategy = UploadNone
        else:
            upload_strategy = UploadAll

        # determine actions
        actions = []
        if args.merge:
            actions.append(ProcessActionMergeWithLast())

        if args.downsize:
            actions.append(ProcessActionDownsize(args.max_size))

        if not actions:
            print("No actions specified, will merge files by default")
            actions.append(ProcessActionMergeWithLast())

        # create run metadata

        run_metadata = InfluConfig(
            fastq_dir=args.in_dir,
            output_dir=args.out_dir,
            name_tag=args.tag,
            uploader=insaflu_upload,
            upload_strategy=upload_strategy,
            actions=actions,
            max_size=args.max_size,
            keep_name=args.keep_names,
            sleep_time=args.sleep,
            deploy_televir=args.televir,
            monitor=args.monitor,
        )

        return run_metadata

    def generate_runners(
        self, run_metadata: InfluConfig
    ) -> Tuple[InsafluFileProcess, TelevirFileProcess]:
        influ_compressor = InsafluFileProcess(
            run_metadata,
        )

        televir_processor = TelevirFileProcess(
            run_metadata,
        )

        return influ_compressor, televir_processor

    def generate_threads(
        self, file_processor: InsafluFileProcess, televir_processor: TelevirFileProcess
    ) -> Tuple[InsafluFileProcessThread, TelevirFileProcessThread]:
        lock = LockWithOwner()
        lock.owner = "A"

        file_processor_task = InsafluFileProcessThread(file_processor, lock)
        televir_processor_task = TelevirFileProcessThread(televir_processor, lock)

        file_processor_task.daemon = True
        televir_processor_task.daemon = True

        return file_processor_task, televir_processor_task

    def monitor_tasks(
        self,
        file_processor_task: InsafluFileProcessThread,
        televir_processor_task: TelevirFileProcessThread,
        args: ArgsClass,
    ):
        if not args.monitor:
            if file_processor_task.counter > 0 and televir_processor_task.counter > 0:
                return False

        if not file_processor_task.is_alive() and not televir_processor_task.is_alive():
            return False

        if file_processor_task.error or televir_processor_task.error:
            return False

        return True

    def run(self):
        args = self.args

        run_metadata = self.setup_config(args)

        influ_compressor, televir_processor = self.generate_runners(run_metadata)

        file_processor_task, televir_processor_task = self.generate_threads(
            influ_compressor, televir_processor
        )

        signal.signal(signal.SIGINT, signal_handler)

        loop = True

        try:
            file_processor_task.start()
            televir_processor_task.start()

            while loop:
                new_loop = self.monitor_tasks(
                    file_processor_task, televir_processor_task, args
                )
                loop = new_loop

                time.sleep(2)

            print("Waiting for tasks to finish...")

        except Exception as e:
            print("Error in main thread.")
            print(e)

        finally:
            print("Stopping tasks...")

            file_processor_task.stop()
            televir_processor_task.stop()

            file_processor_task.join()
            televir_processor_task.join()

            print("Done!")

            sys.exit(0)



import configparser
import subprocess
import sys
from abc import ABC, abstractmethod
from typing import Optional

import paramiko


class Connector(ABC):

    @abstractmethod
    def __init__(self):
        pass

    @abstractmethod
    def test_connection(self):
        pass

    @abstractmethod
    def execute_command(self, command: str) -> str:
        pass

    @abstractmethod
    def check_file_exists(self, file_path: str) -> bool:
        pass

    @abstractmethod
    def upload_file(self, file_path: str, remote_path: str):
        pass

    @abstractmethod
    def download_file(self, file_path: str, remote_path: str):
        pass

    @abstractmethod
    def __enter__(self):
        pass

    @abstractmethod
    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class ConnectorNull(Connector):

    def __init__(self):
        super().__init__()

    def test_connection(self) -> None:
        pass

    def execute_command(self, command: str) -> str:
        return ""

    def check_file_exists(self, file_path: str) -> bool:
        return False

    def upload_file(self, file_path: str, remote_path: str):
        pass

    def download_file(self, file_path: str, remote_path: str):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class ConnectorParamiko(Connector):

    def __init__(self, config_file: str) -> None:
        super().__init__()
        self.prep_input(config_file)
        self.connect()

    def input_config(self, config_file: str):

        config = configparser.ConfigParser()
        config.read(config_file)

        username = config["SSH"].get("username", None)
        ip_address = config["SSH"].get("ip_address", None)
        rsa_key_path = config["SSH"].get("rsa_key", None)

        if username is None or ip_address is None or rsa_key_path is None:
            raise ValueError("Config file is missing values")

        return username, ip_address, rsa_key_path

    def input_user(self):
        username = input("username: ")
        ip_address = input("ip_address: ")
        rsa_key_path = input("rsa_key (full path): ")

        return username, ip_address, rsa_key_path

    def prep_input(self, config_file: Optional[str] = None):
        if config_file is None:
            username, ip_address, rsa_key_path = self.input_user()
        try:
            username, ip_address, rsa_key_path = self.input_config(config_file)
        except FileNotFoundError:
            print("Config file not found, please input manually")
            username, ip_address, rsa_key_path = self.input_user()

        self.username = username
        self.ip_address = ip_address
        self.rsa_key_path = rsa_key_path

    def connect(self):

        paramiko_rsa_key = paramiko.RSAKey.from_private_key_file(
            self.rsa_key_path)
        self.rsa_key = paramiko_rsa_key

        self.conn = paramiko.SSHClient()
        self.conn.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        self.test_connection()

    def __enter__(self):

        try:

            self.conn.connect(
                hostname=f"{self.ip_address}",
                username=f"{self.username}",
                pkey=self.rsa_key
            )

        except paramiko.ssh_exception.SSHException as error:
            print("SSH connection error")
            sys.exit(1)

        except KeyboardInterrupt:
            print("Keyboard interrupt")
            sys.exit(1)

        except Exception as error:
            print("Unknown error")
            print(error)

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.conn.close()

    def test_connection(self) -> None:
        """
        test using paramiko using rsa key. If successful, close connection, else exit
        """

        try:
            self.conn.connect(
                hostname=f"{self.ip_address}",
                username=f"{self.username}",
                pkey=self.rsa_key
            )
            self.conn.close()
        except Exception:
            print("Authentication failed, please verify your credentials")
            sys.exit(1)

    def execute_command(self, command: str) -> str:
        """
        execute command using paramiko"""

        with self as conn:

            stdin, stdout, stderr = self.conn.exec_command(command)
            stdout = stdout.read().decode("utf-8")

            return stdout

    def check_file_exists(self, file_path: str) -> bool:
        """
        check file exists using paramiko"""

        with self as conn:
            stdin, stdout, stderr = self.conn.exec_command(f"ls {file_path}")

            if len(stdout.read().decode("utf-8")) > 0:
                if "cannot access" in stdout.read().decode("utf-8"):
                    return False

                return True

            return False

    def upload_file(self, file_path: str, remote_path: str):
        """
        upload file using paramiko"""
        with self as conn:
            ftp_client = self.conn.open_sftp()
            ftp_client.put(file_path, remote_path)
            ftp_client.close()

    def download_file(self, remote_path: str, file_path: str):
        """
        download file using paramiko"""
        with self as conn:
            ftp_client = self.conn.open_sftp()
            ftp_client.get(remote_path, file_path)
            ftp_client.close()


class ConnectorDocker(Connector):
    """
    Connector for docker containers
    """
    config_image_name = "image"
    televir_username = "flu_user"

    def __init__(self, config_file: str) -> None:
        super().__init__()
        self.prep_input(config_file)
        self.set_interactive(True)

    def set_interactive(self, interactive: bool):
        if interactive:
            self.interactive = "-it"
        else:
            self.interactive = ""

    def input_config(self, config_file: str):
        """
        read config file for docker"""

        config = configparser.ConfigParser()
        config.read(config_file)

        server_name = config["DOCKER"].get(self.config_image_name, None)

        if server_name is None:
            raise ValueError("Config file is missing values")

        return server_name

    def input_user(self):
        """
        input user for docker"""
        server_name = input("server name: ")

        return server_name

    def prep_input(self, config_file: Optional[str] = None):
        """
        prepare input for docker"""
        if config_file is None:
            server_name = self.input_user()
        try:
            server_name = self.input_config(config_file)
        except FileNotFoundError:
            print("Config file not found, please input manually")
            server_name = self.input_user()

        self.server_name = server_name

    def test_connection(self) -> None:
        """
        test using paramiko using rsa key. If successful, close connection, else exit
        """

        try:
            bash_command = f"docker exec {self.interactive} {self.server_name} echo 'test'"

            process = subprocess.Popen(
                bash_command.split(), stdout=subprocess.PIPE)
            output, error = process.communicate()

            if error is not None:
                raise Exception

        except Exception:
            print("Authentication failed, please verify your credentials")
            sys.exit(1)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def execute_command(self, command: str) -> str:
        """
        execute command using in docker container"""

        command = f'su - {self.televir_username} -c "{command}"'

        bash_command = f"docker exec {self.interactive} {self.server_name} {command}"

        # SUBMIT COMMAND STRING WITHOUT SPLITTING
        process = subprocess.Popen(
            bash_command, stdout=subprocess.PIPE, shell=True)

        output, error = process.communicate()

        return output.decode("utf-8")

    def check_file_exists(self, file_path: str) -> bool:
        """
        check file exists in docker container"""

        bash_command = f"docker exec {self.interactive} {self.server_name} ls {file_path}"

        process = subprocess.Popen(
            bash_command.split(), stdout=subprocess.PIPE)
        output, error = process.communicate()

        if len(output.decode("utf-8")) > 0:
            if "cannot access" in output.decode("utf-8"):
                return False

            return True

        return False

    def upload_file(self, file_path: str, remote_path: str):
        """
        upload file using docker cp"""

        bash_command = f"docker cp {file_path} {self.server_name}:{remote_path}"

        process = subprocess.Popen(
            bash_command.split(), stdout=subprocess.PIPE)
        output, error = process.communicate()

    def download_file(self, file_path: str, remote_path: str):
        """
        download file using docker cp"""

        bash_command = f"docker cp {self.server_name}:{file_path} {remote_path}"

        process = subprocess.Popen(
            bash_command.split(), stdout=subprocess.PIPE)
        output, error = process.communicate()




import logging
import os
import sys
from dataclasses import dataclass
from typing import Optional, Type

from fastq_handler.configs import InputState, OutputDirs, RunParams

from findontime.tables_post import InsafluTables
from findontime.upload_utils import InsafluUpload, UploadStrategy

default_log_handler = logging.StreamHandler(sys.stdout)
default_log_handler.setLevel(logging.INFO)
formatter = logging.Formatter(
    '%(asctime)s - %(message)s')
default_log_handler.setFormatter(formatter)


@dataclass
class InfluParams(RunParams):
    """
    class to hold parameters with defaults
    """
    deploy_televir: bool = False
    monitor: bool = False


@dataclass
class InfluUpload(InputState):
    """
    class to hold input
    """
    uploader: Optional[InsafluUpload] = None
    upload_strategy: Optional[Type[UploadStrategy]] = None


@dataclass
class InfluOutput(OutputDirs):
    tsv_temp_name: str = "televir_metadata.tsv"
    metadata_dirname: str = "metadata_dir"
    logs_dirname: str = "logs"
    db_path: str = "insaflu.db"


@dataclass
class RunConfigMeta(InputState, RunParams, InfluOutput):
    """
    class to hold run config"""

    def __post_init__(self):
        """
        post init
        """
        if self.actions is None:
            self.actions = []

        self.logs_dir = os.path.join(self.output_dir, self.logs_dirname)
        self.output_dir = os.path.abspath(self.output_dir)

        self.metadata_dir = os.path.join(
            self.output_dir, self.metadata_dirname)


@dataclass
class InfluConfig(InfluParams, InfluUpload, InfluOutput):
    """
    TelevirConfig class
    """

    def __post_init__(self):

        self.name_tag = self.name_tag.strip()
        self.uploader = self.uploader
        self.upload_strategy = self.upload_strategy
        self.actions = self.actions

        self.keep_name = self.keep_name
        self.deploy_televir = self.deploy_televir

        self.metadata_dir = os.path.join(
            self.output_dir, self.metadata_dirname)

        self.logs_dir = os.path.join(self.output_dir, self.logs_dirname)
        self.db_path = os.path.join(self.logs_dir, self.db_path)

        os.makedirs(self.metadata_dir, exist_ok=True)
        os.makedirs(self.logs_dir, exist_ok=True)

        self.tables = InsafluTables(self.db_path)

    def setup_db(self):

        self.tables.setup()

    def delete_db(self):
        """
        delete db
        """
        self.tables.drop_database()
